#!D:/Code/python
# -*- coding: utf-8 -*-
# @Time :2022/1/17 10:14
# @Author : NZL
# @File : __init__.py.py
# @Desc :
from .AESUtils import aes_decrypt, aes_encrypt
from .DesUtils import des_decrypt, des_encrypt
